@@ -22,13 +22,16 @@ type Logger struct {
 	labels Labels
 }
 
-// WithLabels returns a logger whose module is the same
-// as this logger and the returned logger will add the
-// specified labels to each log entry.
+// WithLabels returns a logger whose module is the same as this logger and
+// the returned logger will add the specified labels to each log entry.
+// WithLabels only target a specific logger with labels. Children of the logger
+// will not inherit the labels.
+// To add labels to all child loggers, use ChildWithLabels.
 func (logger Logger) WithLabels(labels Labels) Logger {
 	if len(labels) == 0 {
 		return logger
 	}
+
 	result := logger
 	result.labels = make(Labels)
 	for k, v := range labels {
@@ -85,6 +88,34 @@ func (logger Logger) ChildWithTags(name string, tags ...string) Logger {
 	return module.context.GetLogger(path, tags...)
 }
 
+// ChildWithLabels returns the Logger whose module name is the composed of this
+// Logger's name and the specified name with the correct associated labels.
+// Adding labels to the child logger will cause all child loggers to also
+// inherit the labels of the parent(s) loggers.
+// For targeting a singular logger with labels, use WithLabels which are not
+// inherited by child loggers.
+func (logger Logger) ChildWithLabels(name string, labels Labels) Logger {
+	module := logger.getModule()
+	path := module.name
+	if path == "" {
+		path = name
+	} else {
+		path += "." + name
+	}
+
+	merged := make(Labels)
+	for k, v := range logger.impl.labels {
+		merged[k] = v
+	}
+	for k, v := range labels {
+		merged[k] = v
+	}
+
+	result := module.context.GetLogger(path)
+	result.impl.labels = merged
+	return result
+}
+
 // Name returns the logger's module name.
 func (logger Logger) Name() string {
 	return logger.getModule().Name()
@@ -128,6 +159,15 @@ func (logger Logger) Logf(level Level, message string, args ...interface{}) {
 	logger.LogCallf(2, level, message, args...)
 }
 
+// LogWithlabelsf logs a printf-formatted message at the given level with extra
+// labels. The given labels will be added to the log entry.
+// A message will be discarded if level is less than the the effective log level
+// of the logger. Note that the writers may also filter out messages that are
+// less than their registered minimum severity level.
+func (logger Logger) LogWithLabelsf(level Level, message string, extraLabels map[string]string, args ...interface{}) {
+	logger.logCallf(2, level, message, extraLabels, args...)
+}
+
 // LogCallf logs a printf-formatted message at the given level.
 // The location of the call is indicated by the calldepth argument.
 // A calldepth of 1 means the function that called this function.
@@ -136,6 +176,12 @@ func (logger Logger) Logf(level Level, message string, args ...interface{}) {
 // Note that the writers may also filter out messages that
 // are less than their registered minimum severity level.
 func (logger Logger) LogCallf(calldepth int, level Level, message string, args ...interface{}) {
+	logger.logCallf(calldepth+1, level, message, nil, args...)
+}
+
+// logCallf is a private method for logging a printf-formatted message at the
+// given level. Used by LogWithLabelsf and LogCallf.
+func (logger Logger) logCallf(calldepth int, level Level, message string, extraLabels map[string]string, args ...interface{}) {
 	module := logger.getModule()
 	if !module.willWrite(level) {
 		return
@@ -171,15 +217,19 @@ func (logger Logger) LogCallf(calldepth int, level Level, message string, args .
 		Timestamp: now,
 		Message:   formattedMessage,
 	}
-	if len(module.tags) > 0 || len(logger.labels) > 0 {
-		entry.Labels = make(Labels)
-		if len(module.tags) > 0 {
-			entry.Labels = make(Labels)
-			entry.Labels[LoggerTags] = strings.Join(module.tags, ",")
-		}
-		for k, v := range logger.labels {
-			entry.Labels[k] = v
-		}
+	entry.Labels = make(Labels)
+	if len(module.tags) > 0 {
+		entry.Labels[LoggerTags] = strings.Join(module.tags, ",")
+	}
+	for k, v := range logger.impl.labels {
+		entry.Labels[k] = v
+	}
+	for k, v := range logger.labels {
+		entry.Labels[k] = v
+	}
+	// Add extra labels if there's any given.
+	for k, v := range extraLabels {
+		entry.Labels[k] = v
 	}
 	module.write(entry)
 }
@@ -204,6 +254,12 @@ func (logger Logger) Infof(message string, args ...interface{}) {
 	logger.Logf(INFO, message, args...)
 }
 
+// InfoWithLabelsf logs the printf-formatted message at info level with extra
+// labels.
+func (logger Logger) InfoWithLabelsf(message string, extraLabels map[string]string, args ...interface{}) {
+	logger.LogWithLabelsf(INFO, message, extraLabels, args...)
+}
+
 // Debugf logs the printf-formatted message at debug level.
 func (logger Logger) Debugf(message string, args ...interface{}) {
 	logger.Logf(DEBUG, message, args...)
